@@ -1,6 +1,7 @@
+# coding=UTF-8
 __copyright__ = "Copyright 2014 Davide Olianas (ubuntupk@gmail.com)."
 
-__license__ = """This file is part of poliBeePsync.
+__license__ = """This f is part of poliBeePsync.
 poliBeePsync is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
@@ -20,20 +21,152 @@ from datetime import datetime, timedelta, tzinfo
 import requests
 import os
 import logging
-from pyparsing import Word, alphanums, alphas, nums, Group, OneOrMore,\
+from pyparsing import Word, alphanums, alphas, nums, Group, OneOrMore, \
     Literal, ParseException
+from threading import Thread
+from signalslot import Signal as sSignal
+
 
 logger = logging.getLogger("polibeepsync.common")
 
 
+# --- Custom Exceptions --- #
+
 class InvalidLoginError(Exception):
-    pass
+    """Exception raised when user code, password or both are wrong."""
+    def __init__(self):
+        pass
 
 
-class CourseNotFoundError(Exception):
-    pass
+# --- Custom Threads --- #
+class LoginThread(Thread):
+    def __init__(self, user):
+        Thread.__init__(self)
+        self.name = 'loginthread'
+        self.user = user
+        self.login_status = sSignal(args=['message'])
+
+    def run(self):
+        try:
+            self.user.logout()
+            self.user.login()
+            if self.user.logged is True:
+                self.login_status.emit(message='Successful login.')
+                logger.info('Successful login.')
+        except InvalidLoginError:
+            self.user.logout()
+            self.login_status.emit(message='Login failed.')
+            logger.error("Login failed.", exc_info=True)
+        except requests.ConnectionError:
+            self.user.logout()
+            self.login_status.emit(message='I can\'t connect to the'
+                                           ' server. Is the Internet'
+                                           ' connection working?')
+            logger.error('Connection error.', exc_info=True)
+        except requests.Timeout:
+            self.user.logout()
+            self.login_status.emit(message="The timeout time has been"
+                                           " reached. Is the Internet"
+                                           " connection working?")
+            logger.error("Timeout error.", exc_info=True)
 
 
+class SyncThread(Thread):
+    def __init__(self, user):
+        Thread.__init__(self)
+        self.name = 'syncthread'
+        self.user = user
+        self.signal = sSignal(args=['message'])
+        self.new = sSignal(args=['course'])
+        self.old = sSignal(args=['course'])
+
+    def run(self):
+        most_recent = self.user.get_online_courses()
+        last = self.user.available_courses
+        new = most_recent - last
+        removable = last - most_recent
+        if len(removable) > 0:
+            self.signal.emit(message='The following courses have'
+                                     ' been removed because they '
+                                     'aren\'t available online: {}'
+                             .format(removable))
+            for course in removable:
+                self.old.emit(course=course)
+        if len(new) > 0:
+            for course in new:
+                course.save_folder_name = course.simplify_name(course.name)
+                self.signal.emit(message='A new course '
+                                         'was found: {}'.format(course))
+                self.new.emit(course=course)
+        if len(new) == 0:
+            self.signal.emit(message='No new courses found.')
+            logger.info('No new courses found.')
+        self.user.sync_available_courses(most_recent)
+        logger.info('User object changed')
+
+
+class DownloadThread(object):
+    def __init__(self, user, topdir):
+        self.user = user
+        self.topdir = topdir
+        self.download_signal = sSignal(args=['course'])
+        self.initial_sizes = sSignal(args=['course'])
+        self.data_signal = sSignal(args=['data'])
+
+    def start(self):
+        t = Thread(target=self.run)
+        t.start()
+
+    def run(self):
+        for course in self.user.available_courses:
+            subdir = course.save_folder_name
+            if course.sync is True:
+                try:
+                    outdir = os.path.join(self.topdir, subdir)
+                    os.makedirs(outdir, exist_ok=True)
+                    self.user.update_course_files(course)
+                    syncthese = []
+                    savedhere = os.path.join(self.topdir,
+                                             course.save_folder_name)
+                    needsync = need_syncing(course.documents,
+                                            savedhere,
+                                            syncthese)
+
+                    syncsize = total_size(needsync)
+                    print('****SYNCSIZE: ', syncsize)
+                    alreadysynced = course.total_file_size - syncsize
+                    print('****ALREADYSYNCED ', alreadysynced)
+                    course.downloaded_size = alreadysynced
+                    print('****DOWNLOADED SIZE setting to ',
+                          course.downloaded_size)
+                    self.initial_sizes.emit(course=course)
+
+                    self.user.save_files(course, needsync,
+                                         self.download_signal,
+                                         self.data_signal)
+                    # adesso ogni f di syncthese ha la data di download
+                    # aggiornata, ma deve essere scritto su f
+                    logger.info("Synced files for {}".format(course.name))
+                except InvalidLoginError:
+                    self.user.logout()
+                    logger.info("Login failed.", exc_info=True)
+                except requests.ConnectionError:
+                    self.user.logout()
+                    # self.signal_error.sig.emit('I can\'t connect to'
+                    # ' the server. Is the'
+                    # ' Internet connection'
+                    # ' working?')
+                    logger.error('Connection error.', exc_info=True)
+                except requests.Timeout:
+                    self.user.logout()
+                    # self.signal_error.sig.emit("The timeout time has"
+                    # " been reached. Is the"
+                    # " Internet connection"
+                    # " working?")
+                    logger.error("Timeout error.", exc_info=True)
+
+
+# --- "Core" classes here --- #
 class GMT1(tzinfo):
     def utcoffset(self, dt):
         return timedelta(hours=1) + self.dst(dt)
@@ -122,8 +255,10 @@ class Course(GenericSet):
         self.name = name
         self.documents_url = documents_url
         self.sync = sync
-        self.documents = ""
+        self.documents = Folder('root', self.documents_url)
         self.save_folder_name = ""
+        self.total_file_size = 0  # in bytes
+        self.downloaded_size = 0  # in bytes
 
     def simplify_name(self, name):
         simple = name
@@ -168,12 +303,21 @@ class CourseFile(object):
         self.name = name
         self.url = url
         self.last_online_edit_time = last_online_edit_time
-        self.local_creation_time = datetime(1990, 1, 1, 1, 1,
-                                            tzinfo=gmt1)
+        self.local_creation_time = None
+        self.size = 0  # in bytes
 
     def __hash__(self):
         return hash(self.name)
 
+    def __repr__(self):
+        return self.name
+
+    def __eq__(self, other):
+        if os.path.splitext(self.name)[0] == os.path.splitext(other.name)[0]:
+            return True
+        else:
+            return False
+
 
 class Folder(object):
     def __init__(self, name, url):
@@ -182,6 +326,102 @@ class Folder(object):
         self.files = []
         self.folders = []
 
+    def __repr__(self):
+        return self.name + " folder"
+
+    def __eq__(self, other):
+        if self.name == other.name:
+            return True
+        else:
+            return False
+
+
+def total_size(listoffiles):
+    total = 0
+    for f, path in listoffiles:
+        logger.debug('Size of {}: {} bytes'.format(f.name, f.size))
+        total += f.size
+    logger.debug('Total size: {}'.format(total))
+    return total
+
+
+def folder_total_size(parentfolder, sizes):
+    for f in parentfolder.files:
+        print('il f ', f.name, ' è grosso ', f.size)
+        print('prima di aggiungere, size è ', sizes)
+        sizes.append(f.size)
+        print('dopo operazione, size è ', sizes)
+    for folder in parentfolder.folders:
+        print('sto controllando la dimensione della sottocartella ',
+              folder.name)
+        folder_total_size(folder, sizes)
+        # viene passata sempre la stessa dimensione della cartella più in alto
+    return sizes
+
+
+def synclocalwithonline(local, online):
+    """Modifies local in order to reflect changes from online"""
+    for f in online.files:
+        if f not in local.files:
+            local.files.append(f)
+        else:
+            ind = local.files.index(f)
+            local.files[ind].last_online_edit_time = f.last_online_edit_time
+    oldfiles = [f for f in local.files if f not in online.files]
+    cleanfiles = [f for f in local.files if f not in oldfiles]
+    local.files = cleanfiles
+    old = [f for f in local.folders if f not in online.folders]
+    clean = [f for f in local.folders if f not in old]
+    local.folders = clean
+    for folder in online.folders:
+        if folder not in local.folders:
+            local.folders.append(folder)
+    for folder in online.folders:
+        ind = local.folders.index(folder)
+        synclocalwithonline(local.folders[ind], folder)
+    return local
+
+
+def need_syncing(folder, parent_folder, syncthese):
+    """Return a flat list with files to download
+
+    Each element is a tuple like this
+    (f, path)
+
+    filename is the f as scraped from the web (its name can be with or
+    without the extension)
+    path is the absolute path of the folder in which the f should be
+    downloaded
+    """
+    print('calling with folder=', folder.name, ', parent folder= ',
+          parent_folder, ", lunghezza syncthese = ", len(syncthese))
+    # basenames contains the names of files without extension (this is used
+    # later because the website sometimes doesn't show the f extension)
+    basenames = []
+    if os.path.exists(parent_folder):
+        basenames = [os.path.splitext(os.path.basename(f))[0]
+                     for f in os.listdir(parent_folder)
+                     if os.path.isfile(os.path.join(parent_folder, f))]
+    for f in folder.files:
+        # print(f.local_creation_time, f.last_online_edit_time)
+        simplename = os.path.join(parent_folder, f.name)
+        if f.local_creation_time is None:
+            print('data None')
+            syncthese.append((f, parent_folder))
+        elif f.local_creation_time < f.last_online_edit_time:
+            print('creazione < online')
+            syncthese.append((f, parent_folder))
+        elif not os.path.exists(simplename) and f.name not in basenames:
+            print('scommetto che penso che esistono quelli senza estensione')
+            print('f.name = ', f.name)
+            print('altrimenti')
+            print('non esiste e allora aggiungo')
+            syncthese.append((f, parent_folder))
+    for f in folder.folders:
+        new_parent = os.path.join(parent_folder, f.name)
+        need_syncing(f, new_parent, syncthese)
+    return syncthese
+
 
 class User(object):
     loginurl = 'https://beep.metid.polimi.it/polimi/login'
@@ -196,13 +436,17 @@ class User(object):
         self.courses_url = ""
         self.available_courses = Courses()
         self.root_save_folder = ""
+        self.chunk_download = sSignal(args=['course'])
+        # self.chunk_download.connect(self.print_chunk)
 
+    # def print_chunk(self, **kwargs):
+    # print(kwargs['chunk_size'])
 
     def logout(self):
         """Logout.
 
         It re-creates a session and sets :attr:`logged` to ``False``."""
-        del (self.session)
+        del self.session
         self.session = requests.Session()
         # self.session.cookies.clear()
         self.logged = False
@@ -227,10 +471,10 @@ class User(object):
         return response
 
     def get_file(self, url):
-        """Use this method to get a file.
+        """Use this method to get a f.
 
         It will check if the session is expired, and re-login if necessary.
-        The file bytes can be accessed with the :attr:`content` attribute
+        The f bytes can be accessed with the :attr:`content` attribute
 
         >>> user = User('username', 'password')
         >>> response = user.get_file('url_to_file', timeout=5, verify=True)
@@ -240,13 +484,14 @@ class User(object):
         Returns:
             response (requests.Response): a :class:`requests.Response` object
         """
-        response = self.session.get(url, timeout=5, verify=True)
+        response = self.session.get(url, timeout=5, verify=True, stream=True)
         if len(response.history) > 0:
             # it means that we've been redirected to the login page
             logger.info("The session has expired. Logging-in again...")
             self.logout()
             self.login()
-            response = self.session.get(url, timeout=5, verify=True)
+            response = self.session.get(url, timeout=5, verify=True,
+                                        stream=True)
         return response
 
     def _login_first_step(self):
@@ -300,15 +545,14 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER",
                 shibsessionstr = "%s=%s" % (key, cookies[key])
         main_headers = {
             'Cookie': "GUEST_LANGUAGE_ID=en_GB; \
-COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
-                shibsessionstr)
+COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" %
+                      shibsessionstr
         }
         mainpage = self.session.get(
             'https://beep.metid.polimi.it/polimi/login',
             headers=main_headers, timeout=5, verify=True)
         return mainpage
 
-
     def login(self):
         """Try logging in.
 
@@ -319,8 +563,6 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
         Raises:
             InvalidLoginError: when the login fails
         """
-
-
         # switch to english version if we're on the italian site
         first_response = self._login_first_step()
         login_soup = BeautifulSoup(first_response.text)
@@ -347,18 +589,19 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
         """
         link = self.get_page(firstlink).url
         link = link.rstrip('attivita-online-e-avvisi')
-        weird_parameters = ['_20_folderId=0',
-                            '_20_displayStyle=list',
-                            '_20_viewEntries=0',
-                            '_20_viewFolders=0',
-                            '_20_entryEnd=500',
-                            '_20_entryStart=0',
-                            '_20_folderEnd=500',
-                            '_20_folderStart=0',
-                            '_20_viewEntriesPage=1',
-                            'p_p_id=20',
-                            'p_p_lifecycle=0'
-        ]
+        weird_parameters = [
+            '_20_folderId=0',
+            '_20_displayStyle=list',
+            '_20_viewEntries=0',
+            '_20_viewFolders=0',
+            '_20_entryEnd=500',
+            '_20_entryStart=0',
+            '_20_folderEnd=500',
+            '_20_folderStart=0',
+            '_20_viewEntriesPage=1',
+            'p_p_id=20',
+            'p_p_lifecycle=0'
+            ]
         link = link + 'documenti-e-media?' + "&".join(weird_parameters)
         course = Course(name, link)
         logger.debug('Course found: {}'.format(course.name))
@@ -382,9 +625,9 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
         # we only need year to parse for real courses
         year = Group("[" + OneOrMore(
             Word(nums, exact=4) + "-" + Word(nums, exact=2)) + "]")
-        #bracketed = Group("[" + OneOrMore(Word(printables, " ")) + "]")
-        #middle = ~bracketed + OneOrMore(Word(alphas))
-        #grammar = year.suppress() + Literal("-").suppress() + middle
+        # bracketed = Group("[" + OneOrMore(Word(printables, " ")) + "]")
+        # middle = ~bracketed + OneOrMore(Word(alphas))
+        # grammar = year.suppress() + Literal("-").suppress() + middle
         grammar = year
         for course in raw_courses:
             firstlink = course.td.a['href']
@@ -452,9 +695,13 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
                 self.available_courses.remove(elem)
 
     def update_course_files(self, course):
-        rootfolder = self.find_files_and_folders(course.documents_url,
-                                                 'rootfolder')
-        course.documents = rootfolder
+        online = self.find_files_and_folders(course.documents_url,
+                                             'rootfolder')
+        synclocalwithonline(course.documents, online)
+        sizes = []
+        course.total_file_size = sum(folder_total_size(course.documents,
+                                                        sizes))
+        print('****DIMENSIONE TOTALE: ', course.total_file_size)
 
     def find_files_and_folders(self, link, thisfoldername):
         response = self.get_page(link)
@@ -467,10 +714,10 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
         logger.debug("Tags from which we extract the list of files:"
                      " {}".format(tags))
         rawdates = [elem.parent.parent.parent.next_sibling.next_sibling.
-                        next_sibling.next_sibling.next_sibling.next_sibling
+                    next_sibling.next_sibling.next_sibling.next_sibling
                     for elem in tags]
         last_column = [elem.next_sibling.next_sibling.next_sibling.
-                           next_sibling for elem in rawdates]
+                       next_sibling for elem in rawdates]
 
         folder = Folder(thisfoldername, response.url)
 
@@ -479,7 +726,7 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
             rawdate = rawdates[i]
             day = int(rawdate.text.split(' ')[1].split('/')[0])
             month = int(rawdate.text.split(' ')[1].split('/')[1])
-            year = int(rawdate.text.split(' ')[1].split('/')[2])
+            year = int('20' + rawdate.text.split(' ')[1].split('/')[2])
             hour = int(rawdate.text.split(' ')[2].split('.')[0])
             minute = int(rawdate.text.split(' ')[2].split('.')[1])
             complete_date = datetime(year, month, day, hour, minute,
@@ -488,7 +735,10 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
             elem = download_link[2]
             if elem.text.startswith('  Download ('):
                 link = elem['href']
+                size = int(float(elem.text.strip()[10:-2].replace(".", "").
+                                 replace(",", ".")) * 1024)
                 complete_file = CourseFile(name, link, complete_date)
+                complete_file.size = size
                 folder.files.append(complete_file)
 
             else:
@@ -497,49 +747,23 @@ COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
                 folder.folders.append(subfolder)
         return folder
 
-
-    def save_files(self, masterfolder, out_rootfolder):
-        for coursefile in masterfolder.files:
-            fname = os.path.join(out_rootfolder, coursefile.name)
-            basenames = [os.path.splitext(os.path.basename(f))[0]
-                         for f in os.listdir(out_rootfolder)
-                         if os.path.isfile(os.path.join(out_rootfolder, f))]
-
-            if os.path.exists(fname) and \
-                            coursefile.local_creation_time < \
-                            coursefile.last_online_edit_time:
-                result = self.get_file(coursefile.url)
-                complete_basename = result.headers['Content-Disposition'] \
-                    .split("; ")[1].split("=")[1].strip('"')
-                complete_name = os.path.join(out_rootfolder, complete_basename)
-                with open(complete_name, 'wb') as f:
-                    f.write(result.content)
+    def save_files(self, course, needsync, downloadsignal, datesignal,
+                   chunk_size=512 * 1024):
+        for coursefile, path in needsync:
+            result = self.get_file(coursefile.url)
+            complete_basename = result.headers['Content-Disposition'] \
+                .split("; ")[1].split("=")[1].strip('"')
+            complete_name = os.path.join(path, complete_basename)
+            os.makedirs(path, exist_ok=True)
+            with open(complete_name, 'wb') as f:
+                logger.info('writing into {}'.format(complete_name))
+                for chunk in result.iter_content(chunk_size):
+                    if chunk:
+                        f.write(chunk)
+                        course.downloaded_size += len(chunk)
+                        logger.debug('chunk size: {}'.format(len(chunk)))
+                        downloadsignal.emit(course=course)
                 coursefile.local_creation_time = datetime.now(self.gmt1)
-            elif fname in basenames and \
-                            coursefile.local_creation_time < \
-                            coursefile.last_online_edit_time:
-                result = self.get_file(coursefile.url)
-                complete_basename = result.headers['Content-Disposition'] \
-                    .split("; ")[1].split("=")[1].strip('"')
-                complete_name = os.path.join(out_rootfolder, complete_basename)
-                with open(complete_name, 'wb') as f:
-                    f.write(result.content)
-                coursefile.local_creation_time = datetime.now(self.gmt1)
-            elif not (os.path.exists(fname) and fname in basenames):
-                result = self.get_file(coursefile.url)
-                complete_basename = result.headers['Content-Disposition'] \
-                    .split("; ")[1].split("=")[1].strip('"')
-                complete_name = os.path.join(out_rootfolder, complete_basename)
-                with open(complete_name, 'wb') as f:
-                    f.write(result.content)
-                coursefile.local_creation_time = datetime.now(self.gmt1)
-        if masterfolder.name == "rootfolder":
-            for folder in masterfolder.folders:
-                path = os.path.join(out_rootfolder, folder.name)
-                os.makedirs(path, exist_ok=True)
-                self.save_files(folder, path)
-        else:
-            for folder in masterfolder.folders:
-                path = os.path.join(out_rootfolder, folder.name)
-                os.makedirs(path, exist_ok=True)
-                self.save_files(folder, path)
\ No newline at end of file
+                # we emit another signal here so that we can save to f
+                # the updated local creation time
+                datesignal.emit(data=(course, coursefile, path))
