@@ -15,20 +15,21 @@ You should have received a copy of the GNU General Public License
 along with poliBeePsync. If not, see <http://www.gnu.org/licenses/>.
 """
 
-from requests import ConnectionError, Timeout
+import requests
 from appdirs import user_config_dir, user_data_dir
 import os
 import pickle
 import sys
 
 from PySide.QtCore import (QThread, QObject, Signal, QAbstractTableModel,
-                           QModelIndex, Qt, Slot, QTimer, QLocale, QPoint)
+                           QModelIndex, Qt, Slot, QTimer, QLocale)
 from PySide.QtGui import (QApplication, QWidget, QTextCursor,
                           QMenu, QAction, QFileDialog,
                           QVBoxLayout, QLabel, QSystemTrayIcon,
                           qApp, QDialog)
 
-from polibeepsync.common import User, InvalidLoginError
+from polibeepsync.common import User, InvalidLoginError, Folder, Course, \
+    DownloadThread
 from polibeepsync.cmdlineparser import create_parser
 from polibeepsync.ui_resizable import Ui_Form
 from polibeepsync import filesettings
@@ -41,12 +42,13 @@ parser = create_parser()
 args = parser.parse_args()
 
 # set debug levels
-LEVELS = {'notset': logging.NOTSET,
-          'debug': logging.DEBUG,
-          'info': logging.INFO,
-          'warning': logging.WARNING,
-          'error': logging.ERROR,
-          'critical': logging.CRITICAL,
+LEVELS = {
+    'notset': logging.NOTSET,
+    'debug': logging.DEBUG,
+    'info': logging.INFO,
+    'warning': logging.WARNING,
+    'error': logging.ERROR,
+    'critical': logging.CRITICAL,
 }
 
 level_name = 'notset'
@@ -71,11 +73,6 @@ logger.addHandler(handler)
 commonlogger.addHandler(handler)
 
 
-
-
-
-
-
 def read(*names, **kwargs):
     with open(
             os.path.join(os.path.dirname(__file__), *names),
@@ -104,53 +101,8 @@ class CoursesSignal(QObject):
     sig = Signal(list)
 
 
-class DownloadThread(QThread):
-    def __init__(self, user, topdir, parent=None):
-        QThread.__init__(self, parent)
-        self.exiting = False
-        self.course_finished = MySignal()
-        self.signal_error = MySignal()
-        self.user = user
-        self.topdir = topdir
-        self.dumpuser = MySignal()
-
-    def run(self):
-        while self.exiting == False:
-            for course in self.user.available_courses:
-                subdir = course.save_folder_name
-                if course.sync is True:
-                    try:
-                        self.user.update_course_files(course)
-                        outdir = os.path.join(self.topdir, subdir)
-                        os.makedirs(outdir, exist_ok=True)
-                        rootdir = self.user.find_files_and_folders(
-                            course.documents_url, 'rootfolder')
-                        self.user.save_files(rootdir, outdir)
-                        self.dumpuser.sig.emit('User object changed')
-                        text = "Synced files for {}".format(course.name)
-                        self.course_finished.sig.emit(text)
-                    except InvalidLoginError:
-                        self.user.logout()
-                        self.exiting = True
-                        self.signal_error.sig.emit('Login failed.')
-                        logger.info("Login failed.", exc_info=True)
-                    except ConnectionError:
-                        self.user.logout()
-                        self.exiting = True
-                        self.signal_error.sig.emit('I can\'t connect to'
-                                                   ' the server. Is the'
-                                                   ' Internet connection'
-                                                   ' working?')
-                        logger.error('Connection error.', exc_info=True)
-                    except Timeout:
-                        self.user.logout()
-                        self.exiting = True
-                        self.signal_error.sig.emit("The timeout time has"
-                                                   " been reached. Is the"
-                                                   " Internet connection"
-                                                   " working?")
-                        logger.error("Timeout error.", exc_info=True)
-            self.exiting = True
+class DownloadChunkSignal(QObject):
+    sig = Signal(Course, int)
 
 
 class LoginThread(QThread):
@@ -163,7 +115,7 @@ class LoginThread(QThread):
 
     def run(self):
         logger.info('Logging in.')
-        while self.exiting == False:
+        while not self.exiting:
             try:
                 self.user.logout()
                 self.user.login()
@@ -176,14 +128,14 @@ class LoginThread(QThread):
                 self.exiting = True
                 self.signal_error.sig.emit('Login failed.')
                 logger.error("Login failed.", exc_info=True)
-            except ConnectionError:
+            except requests.ConnectionError:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit('I can\'t connect to the'
                                            ' server. Is the Internet'
                                            ' connection working?')
                 logger.error('Connection error.', exc_info=True)
-            except Timeout:
+            except requests.Timeout:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit("The timeout time has been"
@@ -203,7 +155,7 @@ class RefreshCoursesThread(QThread):
         self.user = user
 
     def run(self):
-        while self.exiting == False:
+        while not self.exiting:
             most_recent = self.user.get_online_courses()
             last = self.user.available_courses
             new = most_recent - last
@@ -239,7 +191,7 @@ class CoursesListModel(QAbstractTableModel):
         return len(self.courses)
 
     def columnCount(self, parent=QModelIndex()):
-        return 3
+        return 4
 
     def insertRows(self, position, rows, newcourse, parent=QModelIndex()):
         self.beginInsertRows(parent, position, position + rows - 1)
@@ -260,8 +212,8 @@ class CoursesListModel(QAbstractTableModel):
             flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
             return flags
         elif index.column() == 1:
-            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable \
-                    | Qt.ItemIsUserCheckable
+            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | \
+                Qt.ItemIsSelectable | Qt.ItemIsUserCheckable
             return flags
         else:
             return Qt.ItemIsEnabled
@@ -280,7 +232,6 @@ class CoursesListModel(QAbstractTableModel):
                 return True
         return False
 
-
     def data(self, index, role=Qt.DisplayRole):
         if role == Qt.DisplayRole:
             if index.column() == 0:
@@ -289,6 +240,10 @@ class CoursesListModel(QAbstractTableModel):
                 return self.courses[index.row()].sync
             if index.column() == 2:
                 return self.courses[index.row()].save_folder_name
+            if index.column() == 3:
+                dw = self.courses[index.row()].downloaded_size
+                total = self.courses[index.row()].total_file_size
+                return (dw, total)
         elif role == Qt.CheckStateRole:
             return None
 
@@ -300,6 +255,8 @@ class CoursesListModel(QAbstractTableModel):
                 return "Sync"
             elif col == 2:
                 return "Save as"
+            elif col == 3:
+                return "Download %"
 
 
 class MainWindow(QWidget, Ui_Form):
@@ -340,9 +297,10 @@ class MainWindow(QWidget, Ui_Form):
 
         self.downloadthread = DownloadThread(self.user,
                                              self.settings['RootFolder'])
-        self.downloadthread.dumpuser.sig.connect(self.dumpUser)
-        self.downloadthread.course_finished.sig.connect(self.myStream_message)
-        self.downloadthread.signal_error.sig.connect(self.myStream_message)
+        self.downloadthread.download_signal.connect(
+            self.update_course_download)
+        self.downloadthread.initial_sizes.connect(self.setinizialsizes)
+        self.downloadthread.data_signal.connect(self.update_file_localtime)
 
         self.userCode.setText(str(self.user.username))
         self.userCode.textEdited.connect(self.setusercode)
@@ -372,6 +330,54 @@ class MainWindow(QWidget, Ui_Form):
 
         self.changeRootFolder.clicked.connect(self.chooserootdir)
 
+    def _update_time(self, folder, file, path_list):
+        print('inside ', folder.name)
+        print('path_list: ', path_list)
+        while len(path_list) > 0:
+            namegoto = path_list.pop(0)
+            print('namegoto: ', namegoto)
+            # perché a volte è vuoto?
+            if namegoto != "":
+                fakefolder = Folder(namegoto, 'fake')
+                print('contained folders: ', folder.folders)
+                ind = folder.folders.index(fakefolder)
+                goto = folder.folders[ind]
+                self._update_time(goto, file, path_list)
+        if file in folder.files:
+            ind = folder.files.index(file)
+            thisfile = folder.files[ind]
+            thisfile.local_creation_time = file.local_creation_time
+            self.dumpUser()
+
+    def update_file_localtime(self, data, **kwargs):
+        course, coursefile, path = data
+        rootpath = os.path.join(self.settings['RootFolder'],
+                                course.save_folder_name)
+        if path.startswith(rootpath):
+            partial = path[len(rootpath):]
+        path_list = partial.split(os.path.sep)
+        self._update_time(course.documents, coursefile, path_list)
+
+    def update_course_download(self, course, **kwargs):
+        logger.info('download size updated')
+        if course in self.user.available_courses:
+            updating = self.user.available_courses[course.name]
+            updating.downloaded_size = course.downloaded_size
+            row = self.courses_model.courses.index(updating)
+            where = self.courses_model.index(row, 3)
+            self.courses_model.dataChanged.emit(where, where)
+            self.dumpUser()
+
+    def setinizialsizes(self, course, **kwargs):
+        if course in self.user.available_courses:
+            updating = self.user.available_courses[course.name]
+            updating.downloaded_size = course.downloaded_size
+            updating.total_file_size = course.total_file_size
+            row = self.courses_model.courses.index(updating)
+            where = self.courses_model.index(row, 3)
+            self.courses_model.dataChanged.emit(where, where)
+            self.dumpUser()
+
     def syncnewcourses(self, newlist):
         if self.settings['SyncNewCourses'] == 'True':
             for elem in newlist:
@@ -421,7 +427,7 @@ class MainWindow(QWidget, Ui_Form):
     # if state == 2:
     # self.settings['NotifyNewCourses'] = 'True'
     # else:
-    #        self.settings['NotifyNewCourses'] = 'False'
+    # self.settings['NotifyNewCourses'] = 'False'
     #    filesettings.settingsToFile(self.settings, self.settings_path)
 
     @Slot(int)
@@ -443,7 +449,6 @@ class MainWindow(QWidget, Ui_Form):
         self.settings['RootFolder'] = path
         filesettings.settingsToFile(self.settings, self.settings_path)
 
-
     def chooserootdir(self):
         currentdir = self.settings['RootFolder']
         flags = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
@@ -465,7 +470,6 @@ class MainWindow(QWidget, Ui_Form):
                 self.myStream_message)
             self.downloadthread.signal_error.sig.connect(self.myStream_message)
 
-
     def setusercode(self):
         newcode = self.userCode.text()
         self.user.username = newcode
@@ -481,7 +485,6 @@ class MainWindow(QWidget, Ui_Form):
             logger.error('OSError raised while trying to write the User'
                          'instance to disk.', exc_info=True)
 
-
     def setpassword(self):
         newpass = self.password.text()
         self.user.password = newpass
@@ -511,7 +514,6 @@ class MainWindow(QWidget, Ui_Form):
             index = self.courses_model.courses.index(elem)
             self.courses_model.removeRows(index, 1)
 
-
     def dumpUser(self):
         # we don't use the message...
         with open(os.path.join(user_data_dir(self.appname),
@@ -537,10 +539,7 @@ class MainWindow(QWidget, Ui_Form):
 
     def do_syncfiles(self):
         self.refreshcoursesthread.finished.disconnect(self.do_syncfiles)
-        if not self.downloadthread.isRunning():
-            self.downloadthread.exiting = False
-            self.downloadthread.start()
-
+        self.downloadthread.start()
 
     @Slot(str)
     def myStream_message(self, message):
